@@ -259,6 +259,7 @@ evma_num_close_scheduled
 
 extern "C" int evma_num_close_scheduled ()
 {
+	ensure_eventmachine("evma_num_close_scheduled");
 	return EventMachine->NumCloseScheduled;
 }
 
@@ -282,6 +283,16 @@ extern "C" const unsigned long evma_create_unix_domain_server (const char *filen
 	return EventMachine->CreateUnixDomainServer (filename);
 }
 
+/***********************
+evma_attach_sd
+************************/
+
+extern "C" const unsigned long evma_attach_sd (int sd)
+{
+	ensure_eventmachine("evma_attach_sd");
+	return EventMachine->AttachSD (sd);
+}
+
 /*************************
 evma_open_datagram_socket
 *************************/
@@ -642,7 +653,6 @@ extern "C" int evma_get_max_timer_count()
 	return EventMachine_t::GetMaxTimerCount();
 }
 
-
 /************************
 evma_set_max_timer_count
 ************************/
@@ -660,6 +670,21 @@ extern "C" void evma_set_max_timer_count (int ct)
 	EventMachine_t::SetMaxTimerCount (ct);
 }
 
+/******************
+evma_get/set_simultaneous_accept_count
+******************/
+
+extern "C" void evma_set_simultaneous_accept_count (int count)
+{
+	EventMachine_t::SetSimultaneousAcceptCount(count);
+}
+
+extern "C" int evma_get_simultaneous_accept_count()
+{
+	return EventMachine_t::GetSimultaneousAcceptCount();
+}
+
+
 /******************
 evma_setuid_string
 ******************/
@@ -751,8 +776,11 @@ extern "C" int evma_send_file_data_to_connection (const unsigned long binding, c
 
 	ensure_eventmachine("evma_send_file_data_to_connection");
 
+#if defined(OS_WIN32)
+	int Fd = open (filename, O_RDONLY|O_BINARY);
+#else
 	int Fd = open (filename, O_RDONLY);
-
+#endif
 	if (Fd < 0)
 		return errno;
 	// From here on, all early returns MUST close Fd.
@@ -774,7 +802,6 @@ extern "C" int evma_send_file_data_to_connection (const unsigned long binding, c
 		return -1;
 	}
 
-
 	r = read (Fd, data, filesize);
 	if (r != filesize) {
 		int e = errno;
